///////////////////////////////////////////////////////////////////////////////
//
// IBM VisualAge C++
// Licensed Materials - Property of IBM
//
// Copyright IBM Corp. 1998,2000. All Rights Reserved
//
// DISCLAIMER:
// This sample code is subject to the license and terms and conditions 
// found in the license.txt file in this directory. It is not part of any 
// standard IBM product and is provided to you solely for the purpose of 
// assisting you in the development of your applications. The code is 
// provided 'AS IS', without warranty or condition of any kind. IBM shall 
// not be liable for any damages arising out of your use of the sample code, 
// even if advised of the possibility of such damages.
//
///////////////////////////////////////////////////////////////////////////////

#define INCL_GPI

#include "Descriptor.h"
#include "DrawingPort.h"
#include "FileLocation.h"
#include "GraphicText.h"
#include "Object.h"
#include "Pane.h"
#include "PaneManager.h"
#include "Platform.h"
#include "Project.h"
#include "Resource.h"
#include "ResourceManager.h"
#include "Utility.h"
#include "WindowManager.h"
#include "Workbook.h"

#include "metrics.h"

#include "Port.t"


static const char *fontFace = "helvetica";
static const unsigned int fontSize = 10;
static IColor titleColor(IColor::black);
static IColor metricsColor(IColor::darkBlue);


//////////////////////////////////////////////////////////////////////////////
// DetailMetricsPort
//////////////////////////////////////////////////////////////////////////////

class DetailMetricsPort : public UI_DrawingPort
{
public:

  DetailMetricsPort		(UI_Pane &pane, const IRectangle &rectangle);

  virtual UI_PortNegotiator &   negotiator                      ();

  virtual void                  setObject                       (const UI_Object &object);

  static Boolean                canAccept                       (const UI_Descriptor &descriptor);
  static UI_Port &              construct                       (UI_Pane &pane, const UI_Object &object, const IRectangle &rectangle);
  static IString                name                            (Boolean translatable);
};

#pragma define(UI_PortNegotiatorFor<class DetailMetricsPort>)

DetailMetricsPort::DetailMetricsPort(UI_Pane &pane, const IRectangle &rectangle) :
  UI_DrawingPort(pane, rectangle)
{
}

UI_PortNegotiator &DetailMetricsPort::negotiator()
{
  return UI_PortNegotiatorFor<class DetailMetricsPort>::instance();
}

void DetailMetricsPort::setObject(const UI_Object &object)
  {

  UI_Port::setObject(object);
 
  if (!fetch())
     {
     UI_GraphicText *text = new UI_GraphicText(fontFace,fontSize);

     CS_Declaration *dec = UI_Cast<CS_Declaration>(object);
     if (dec == NULL) 
        {
        *text << "Null declaration. " << UI_GraphicText::endl;
        store(text);
        return;
        } 
 
     CS_ClassDeclaration *classDecl = dec->asClassDeclaration();
     if (classDecl)
        {
        UI_GraphicList *list                  = UI_New(UI_GraphicList)();
        UI_GraphicInteractor *interactor      = UI_New(UI_GraphicInteractor)();

        *text << text->normalBackgroundColor() << interactor;
        *text << titleColor
              << "Class metrics:" << UI_GraphicText::endl;
        {
        UI_GraphicText::Nest firstNest(*text, *interactor, 0);
        UI_GraphicText::Nest nest (*text, *list, 2);
        *text << metricsColor;
        int i;
        for (i = 0; i < numMetrics; i++)
           {
           *text << getMetricsString((Metric)i) << UI_New(UI_GraphicColumn)(UI_Graphic::leftJustify) << ":"
                 << UI_GraphicText::Space()    << MetricsCalculator::computeMetricValue(*classDecl, (Metric) i)
                 << UI_GraphicText::endl;
           }

        list->layout();
        } // end block scope
        store (text);
	} // if (classDecl)
     else
        { 
        store (0);
        }
     } // if(!fetch)
  }


Boolean DetailMetricsPort::canAccept(const UI_Descriptor &descriptor)
{
  return (descriptor >= UI_DescriptorFor<CS_ClassDeclaration>::descriptor() ||
          descriptor >= UI_DescriptorFor<CS_FunctionDeclaration>::descriptor());
}


UI_Port &DetailMetricsPort::construct(UI_Pane &pane, const UI_Object &object, const IRectangle &rectangle)
{
  UI_Port *port = new DetailMetricsPort(pane, rectangle);
  port->setObject(object);
  return *port;
}


IString DetailMetricsPort::name(Boolean translatable)
{
  return "Detail Metrics";
}




