///////////////////////////////////////////////////////////////////////////////
//
// IBM VisualAge C++
// Licensed Materials - Property of IBM
//
// Copyright IBM Corp. 1998,2000. All Rights Reserved
//
// DISCLAIMER:
// This sample code is subject to the license and terms and conditions 
// found in the license.txt file in this directory. It is not part of any
// standard IBM product and is provided to you solely for the purpose of 
// assisting you in the development of your applications. The code is 
// provided 'AS IS', without warranty or condition of any kind. IBM shall 
// not be liable for any damages arising out of your use of the sample code, 
// even if advised of the possibility of such damages.
//
///////////////////////////////////////////////////////////////////////////////

#ifndef _METRIC_H
#define _METRIC_H
#include <interface/Atom.h>
#include <interface/BaseSpecifier.h>
#include <interface/ClassDeclaration.h>
#include <interface/CodeStore.h>
#include <interface/Declaration.h>
#include <interface/DeclarationStore.h>
#include <interface/DependencyArc.h>
#include <interface/DependencyNode.h>
#include <interface/FunctionDeclaration.h>
#include <interface/FunctionParameter.h>
#include <interface/Implementation.h>
#include <interface/SourceLocation.h>
#include <interface/SourceRegion.h>
#include <interface/SourceStore.h>
#include <interface/Statement.h>
#include <interface/Token.h>

#include <navigator/DeclarationQueries.h>
#include <navigator/DependencyArcAntecedentNodeIterator.h>
#include <navigator/DepthFirstVisitor.h>
#include <navigator/TokenIterator.h>

#include <services/Array.h>

enum Metric 
{   numberOfMemberFunctionsMetric, 
    numberOfVirtualFunctionsMetric, 
    numberOfPureVirtualFunctionsMetric,
    depthOfInheritanceTreeMetric, 
    numberOfChildrenMetric, 
    numberOfNonTemplateDependentMemberFunctionsMetric,// Not totaled metrics
    numMetrics,                     // Total number of enabled metrics
    couplingBetweenClassesMetric    // This is disabled
};  

// Not totaled:
// - depthOfInheritanceTreeMetric, 
// - numberOfChildrenMetric, 
// - numberOfNonTemplateDependentMemberFunctionsMetric 
//
const int numMetricsValidToTotal = numMetrics - 3;  

char* getMetricsString (Metric m);

class MetricsCalculator 
// Groups all the metrics logic together
// 
   {
public:
     // Returns the number of lines of code for a given file
     static unsigned int numberOfLinesOfCode                          (CS_PrimarySource& ps);  
     static unsigned int numberOfMemberFunctions                      (CS_ClassDeclaration& decl);
     static unsigned int numberOfVirtualFunctions                     (CS_ClassDeclaration& decl);
     static unsigned int numberOfPureVirtualFunctions                 (CS_ClassDeclaration& decl);
     static unsigned int depthOfInheritanceTree                       (CS_ClassDeclaration& decl);
     static unsigned int numberOfChildren                             (CS_ClassDeclaration& decl);
     static unsigned int couplingBetweenClasses                       (CS_ClassDeclaration& decl); //Expensive check!
     static unsigned int numberOfNonTemplateDependentMemberFunctions  (CS_ClassDeclaration& decl);
     static unsigned int computeMetricValue                           (CS_Declaration&, Metric m);
   };

class StatisticsCalculator
// Used to calculate median, minimum, and maximum for a given metric
//
   {
public:
   StatisticsCalculator(CS_GlobalDeclarationStore& gds) : _gds(gds) {}
   ~StatisticsCalculator() {};

   unsigned int median    (Metric m);
   unsigned int minimum   (Metric m);
   unsigned int maximum   (Metric m);
 
private:
   CS_GlobalDeclarationStore& _gds;
   };

class TotalsCollector
// Collects project totals
//
{
public:
   TotalsCollector              (CS_CodeStore* cs);
   int numberOfClasses          ();
   int numberOfLinesOfCode      ();
   int numberOfSpecifiedMetric  (Metric);
private:
  CS_GlobalDeclarationStore& _gds;
  CS_SourceStore&            _store;
};

#endif 
