///////////////////////////////////////////////////////////////////////////////
//
// IBM VisualAge C++
// Licensed Materials - Property of IBM
//
// Copyright IBM Corp. 1998,2000. All Rights Reserved
//
// DISCLAIMER:
// This sample code is subject to the license and terms and conditions 
// found in the license.txt file in this directory. It is not part of any 
// standard IBM product and is provided to you solely for the purpose of 
// assisting you in the development of your applications. The code is 
// provided 'AS IS', without warranty or condition of any kind. IBM shall 
// not be liable for any damages arising out of your use of the sample code, 
// even if advised of the possibility of such damages.
//
///////////////////////////////////////////////////////////////////////////////

#define INCL_GPI

#include "Descriptor.h"
#include "DrawingPort.h"
#include "FileLocation.h"
#include "GraphicText.h"
#include "Object.h"
#include "Pane.h"
#include "PaneManager.h"
#include "Platform.h"
#include "Project.h"
#include "Resource.h"
#include "ResourceManager.h"
#include "Utility.h"
#include "WindowManager.h"
#include "Workbook.h"

#include "metrics.h"
#include "Port.t"


static const char *fontFace = "helvetica";
static const unsigned int fontSize = 10;
static IColor titleColor(IColor::black);
static IColor metricsColor(IColor::darkBlue);

///////////////////////////////////////////////////////////////////////////////
// ProjectMetricsPort
///////////////////////////////////////////////////////////////////////////////

class ProjectMetricsPort : public UI_DrawingPort
{
public:

  ProjectMetricsPort		(UI_Pane &pane, const IRectangle &rectangle);

  virtual UI_PortNegotiator &   negotiator                      ();

  virtual void                  setObject                       (const UI_Object &object);

  static Boolean                canAccept                       (const UI_Descriptor &descriptor);
  static UI_Port &              construct                       (UI_Pane &pane, const UI_Object &object, const IRectangle &rectangle);
  static IString                name                            (Boolean translatable);
};


#pragma define(UI_PortNegotiatorFor<class ProjectMetricsPort>)

ProjectMetricsPort::ProjectMetricsPort(UI_Pane &pane, const IRectangle &rectangle) :
  UI_DrawingPort(pane, rectangle)
{
}

UI_PortNegotiator &ProjectMetricsPort::negotiator()
{
  return UI_PortNegotiatorFor<class ProjectMetricsPort>::instance();
}

void ProjectMetricsPort::setObject(const UI_Object &object)
{
  int i;
  UI_Port::setObject(object);

  if (!fetch())
     {
     UI_GraphicText *text = UI_New(UI_GraphicText)(fontFace,fontSize);
     UI_GraphicList *list = UI_New(UI_GraphicList)();
     UI_GraphicInteractor *interactor = UI_New(UI_GraphicInteractor)();
     
     CS_CodeStore *cs = UI_Cast<CS_CodeStore>(object);
     assume (cs);
     CS_GlobalDeclarationStore& gds = cs->globalDeclarationStore();

     TotalsCollector totals(cs);
     *text << text->normalBackgroundColor() << interactor;
     *text << titleColor
           << "Totals" << UI_GraphicText::endl;
     { 
     UI_GraphicText::Nest firstNest (*text, *interactor, 0);
     UI_GraphicText::Nest nest (*text, *list, 2);
     *text << metricsColor 
           << "Number of classes" << UI_New(UI_GraphicColumn)(UI_Graphic::leftJustify) << ":" 
           << UI_GraphicText::Space() << totals.numberOfClasses() << UI_GraphicText::endl
           << "Number of lines of code" << UI_New(UI_GraphicColumn)(UI_Graphic::leftJustify) << ":" 
           << UI_GraphicText::Space() << totals.numberOfLinesOfCode() << UI_GraphicText::endl;

     for (i = 0; i < numMetricsValidToTotal; i++)
        {
        *text << getMetricsString((Metric)i) << UI_New(UI_GraphicColumn)(UI_Graphic::leftJustify) << ":"
              << UI_GraphicText::Space()     << totals.numberOfSpecifiedMetric((Metric) i) << UI_GraphicText::endl;
        }
      list->layout();
      }
     
     UI_GraphicInteractor *ckInteractor = UI_New(UI_GraphicInteractor)();
     UI_GraphicList *ckList = UI_New(UI_GraphicList)();
     StatisticsCalculator statsCalc(gds); 

      *text << text->normalBackgroundColor() << ckInteractor;
      *text << titleColor 
            << "Statistical Information" << UI_GraphicText::endl;
      {
      UI_GraphicText::Nest firstNest(*text, *ckInteractor, 0);
      UI_GraphicText::Nest nest(*text, *ckList, 2);

      *text << metricsColor;
      for (int i = 0; i < numMetrics; i++)
         {
         *text << getMetricsString((Metric) i) << UI_New(UI_GraphicColumn)(UI_Graphic::leftJustify) << ":" << UI_GraphicText::Space() 
               << "min = "    << statsCalc.minimum((Metric) i) << "," <<  UI_GraphicText::Space()
               << "max = "    << statsCalc.maximum((Metric) i) << "," << UI_GraphicText::Space()
               << "median = " << statsCalc.median((Metric) i)  << UI_GraphicText::endl;
         }
      ckList->layout();
      } // end block scope
      store(text); 
      }
   else 
     {
     store (0);
     }
}


Boolean ProjectMetricsPort::canAccept(const UI_Descriptor &descriptor)
{
  return (descriptor >= UI_DescriptorFor<CS_CodeStore>::descriptor()); 
}


UI_Port &ProjectMetricsPort::construct(UI_Pane &pane, const UI_Object &object, const IRectangle &rectangle)
{
  UI_Port *port = new ProjectMetricsPort(pane, rectangle);
  port->setObject(object);
  return *port;
}


IString ProjectMetricsPort::name(Boolean translatable)
{
  return "Project Metrics";
}
