///////////////////////////////////////////////////////////////////////////////
//
// IBM VisualAge C++
// Licensed Materials - Property of IBM
//
// Copyright IBM Corp. 1998,2000. All Rights Reserved
//
// DISCLAIMER:
// This sample code is subject to the license and terms and conditions 
// found in the license.txt file in this directory. It is not part of any
// standard IBM product and is provided to you solely for the purpose of 
// assisting you in the development of your applications. The code is 
// provided 'AS IS', without warranty or condition of any kind. IBM shall 
// not be liable for any damages arising out of your use of the sample code, 
// even if advised of the possibility of such damages.
//
///////////////////////////////////////////////////////////////////////////////

#define INCL_GPI

#include <iseq.h>

#include "Descriptor.h"
#include "DrawingPort.h"
#include "FileLocation.h"
#include "GraphicText.h"
#include "Object.h"
#include "Pane.h"
#include "PaneManager.h"
#include "Platform.h"
#include "Project.h"
#include "Resource.h"
#include "ResourceManager.h"
#include "Utility.h"
#include "WindowManager.h"
#include "Workbook.h"

#include "metrics.h"
#include "qryCnvas.h"
#include "qryCnvas.hpp"

#include "Port.t"

#define LESS      -1
#define EQUAL      0
#define GREATER    1
#define QUERYBUTTON 15004

static const char *fontFace = "helvetica";
static const unsigned int fontSize = 10;
static IColor titleColor(IColor::black);
static IColor metricsColor(IColor::darkBlue);
static const int PANE_SIZE = 65;


//////////////////////////////////////////////////////////////////////////
// QueryMetricsPort
//////////////////////////////////////////////////////////////////////////

class QueryMetricsPort : public UI_DrawingPort
{
public:

  QueryMetricsPort		(UI_Pane &pane, const IRectangle &rectangle);

  virtual UI_PortNegotiator &   negotiator                      ();

  virtual void                  setObject                       (const UI_Object &object);

  static Boolean                canAccept                       (const UI_Descriptor &descriptor);
  static UI_Port &              construct                       (UI_Pane &pane, const UI_Object &object, const IRectangle &rectangle);
  static IString                name                            (Boolean translatable);
  void                          attach                          (bool isAttached); 
  bool				command				(ICommandEvent &event);

private:
  queryCanvas* _queryCanvas;
  const UI_Object*   _currentObj;
};

QueryMetricsPort::QueryMetricsPort (UI_Pane &pane, const IRectangle &rectangle) :
   UI_DrawingPort(pane, rectangle),
   _queryCanvas(0)
   { 
     _queryCanvas = UI_New(queryCanvas)(WND_queryCanvas, &pane, &pane);
     _queryCanvas->initializePart();
     IComboBox * combo = _queryCanvas->getComboBox1();
     for (int i = 0; i < numMetrics; i++)
       combo->addAsLast(getMetricsString((Metric)i));

     ICommandHandler::handleEventsFor(_queryCanvas);
     pane.addExtension(_queryCanvas, UI_PaneWindow::belowClient, PANE_SIZE, UI_PaneWindow::thinLine);
   }
 
CS_bool compare(unsigned int metricsValue, unsigned int threshold, int comparison)
   {
   if (comparison == LESS)
     return metricsValue < threshold;
   else if (comparison == EQUAL)
     return metricsValue == threshold; 
   else 
     return metricsValue > threshold;
   }
#pragma define(UI_PortNegotiatorFor<class QueryMetricsPort>)


UI_PortNegotiator &QueryMetricsPort::negotiator()
{
  return UI_PortNegotiatorFor<class QueryMetricsPort>::instance();
}

void QueryMetricsPort::setObject(const UI_Object &object)
  {
  UI_Port::setObject(object);
  _currentObj = &object;
 
  }

Boolean QueryMetricsPort::canAccept(const UI_Descriptor &descriptor)
{
  return (descriptor >= UI_DescriptorFor<CS_CodeStore>::descriptor());
}


UI_Port &QueryMetricsPort::construct(UI_Pane &pane, const UI_Object &object, const IRectangle &rectangle)
{
  UI_Port *port = UI_New (QueryMetricsPort)(pane, rectangle);
  port->setObject(object);
  return *port;
}


IString QueryMetricsPort::name(Boolean translatable)
{
  return "Query Metrics";
}

void QueryMetricsPort::attach(bool isAttached)
{
  UI_DrawingPort::attach(isAttached);
  if (isAttached)
     pane()->addExtension(_queryCanvas, UI_PaneWindow::aboveClient,
  		 PANE_SIZE, UI_PaneWindow::thinLine);
  else  
     pane()->removeExtension(_queryCanvas);
}

// Display in sorted order
struct ValueDeclPair 
   {
   ValueDeclPair(unsigned int value, CS_Declaration* p) : metricValue(value), pDecl(p) {}
   unsigned int metricValue;
   CS_Declaration* pDecl;
   };

long valueDeclCompare(ValueDeclPair const& p1, ValueDeclPair const& p2)
   { 
   return (p1.metricValue < p2.metricValue);
   }

typedef ISequence<ValueDeclPair> ValueDeclPairSeq;

bool QueryMetricsPort::command(ICommandEvent& event)
   {
   switch (event.commandId()) 
      {
      case QUERYBUTTON:  
  	 UI_GraphicText *text       = UI_New(UI_GraphicText)();
	 CS_CodeStore   *cs         = UI_Cast<CS_CodeStore>(*_currentObj);
	 long            threshold  = _queryCanvas->getSbtnThreshold()->value();
	 IString         rule       = _queryCanvas->getComboBox1()->text();
	 CS_bool         greater    = _queryCanvas->getRadioButton1()->isSelected();
	 CS_bool         equal      = _queryCanvas->getRbtnEqual()->isSelected();
	 CS_bool         less       = _queryCanvas->getRbtnLess()->isSelected();
	 int             comparison = 0; 
	 CS_GlobalDeclarationStore& gds = cs->globalDeclarationStore();
	 Metric metric;

         for (int i = 0; i < numMetrics; i++)
	    {
            if (rule == IString(getMetricsString((Metric)i)))
               metric = (Metric) i;
            }

	 if (greater)
	   comparison = GREATER;
	 else if (equal)
	   comparison = EQUAL;
	 else
	   comparison = LESS;
         
         ValueDeclPairSeq sortedResults;
         unsigned int metricValue = 0;
	 for (CS_Declaration* decl = gds.firstDeclaration();
              decl;
              decl = gds.nextDeclaration(*decl))
	   {
           // If there isn't a metric that can be computed for this decl, 
           // an exception will be thrown
           try 
              {
	      metricValue = MetricsCalculator::computeMetricValue(*decl, metric);
              }
           catch (...)
              {
              continue;
              }
	   if (compare(metricValue, threshold, comparison))
	      {
              ValueDeclPair pair(metricValue, decl);
              sortedResults.addAsLast(pair);
	      }
	   }
         
         sortedResults.sort(valueDeclCompare);
         ValueDeclPairSeq::Cursor cursor(sortedResults);
         
         // Print out sorted results
         for (cursor.setToFirst();
              cursor.isValid();
              cursor.setToNext())
            {
             ValueDeclPair   pair = sortedResults.elementAt(cursor);
             CS_Declaration* decl =  pair.pDecl;
              
	     UI_GraphicText::In in(*text, objectFor(decl));
             *text << pair.metricValue << UI_GraphicText::Space()   
                   << UI_New(UI_GraphicColumn)(UI_Graphic::leftJustify)                            << UI_New(UI_GraphicIcon)() << decl->signature().str() 
                   << UI_GraphicText::endl;
		     
            }
  	 replaceGraphicText(text);
         break; 
      }
   return CS_true;
   }

